﻿Module Utilities
    Public myConfig As New Settings

    Public Class Settings
        Private Const SettingsFileName As String = "config.xml"

        Public Enum ExistingFileHandling
            Skip = 0
            Overwrite = 1
            Rename = 2
        End Enum

        Public Structure Config
            Dim DownloadDir As String
            Dim NamePattern As String
            Dim UncheckAfterAdd As Boolean
            Dim AutoCheckUpdates As Boolean
            Dim ExistingFileHandling As ExistingFileHandling

        End Structure

        Public Settings As Config

        Sub New()
            Call Load()
        End Sub

        Public Sub SetDefaults()
            Settings.DownloadDir = IO.Path.Combine(GetAppPath, "Downloads")
            Settings.NamePattern = "<artist> - <song> - <album>"
            Settings.UncheckAfterAdd = True
            Settings.AutoCheckUpdates = True
            Settings.ExistingFileHandling = ExistingFileHandling.Skip
        End Sub

        Public Sub Load()
            Call SetDefaults()
            Try
                Dim xd As XDocument = XDocument.Load(IO.Path.Combine(GetAppPath, SettingsFileName))
                Settings.DownloadDir = xd.Root.Element("DownloadDir").Value
                Settings.NamePattern = xd.Root.Element("NamePattern").Value
                Settings.UncheckAfterAdd = xd.Root.Element("UncheckAfterAdd").Value
                Settings.AutoCheckUpdates = xd.Root.Element("AutoCheckUpdates").Value
                Settings.ExistingFileHandling = [Enum].Parse(GetType(ExistingFileHandling), xd.Root.Element("ExistingFileHandling").Value)
            Catch
                Call Save()
            End Try
        End Sub

        Public Sub Save()
            Dim xd As New XDocument(New XElement("SciLorsGrooveDownloaderSettings", _
                                    New XElement("DownloadDir", Settings.DownloadDir), _
                                    New XElement("NamePattern", Settings.NamePattern), _
                                    New XElement("UncheckAfterAdd", Settings.UncheckAfterAdd), _
                                    New XElement("AutoCheckUpdates", Settings.AutoCheckUpdates), _
                                    New XElement("ExistingFileHandling", Settings.ExistingFileHandling.ToString())))
            Try
                xd.Save(IO.Path.Combine(GetAppPath, SettingsFileName))
            Catch ex As UnauthorizedAccessException
                If MsgBox("Error: " & ex.Message & vbNewLine & vbNewLine & "You may have to run this application under administration rights to change the settings." & vbNewLine & "Start as administrator?", MsgBoxStyle.YesNo) = MsgBoxResult.Yes Then
                    Dim info As New ProcessStartInfo
                    info.FileName = System.Reflection.Assembly.GetExecutingAssembly().GetName().CodeBase
                    info.UseShellExecute = True
                    info.Verb = "runas"
                    Try
                        Process.Start(info)
                    Catch
                        MsgBox("No administration rights, closing...", MsgBoxStyle.Critical)
                    End Try
                    Application.Current.Shutdown()
                End If
            End Try
        End Sub

        Public Function GetAppPath() As String
            GetAppPath = System.IO.Path.GetDirectoryName(System.Reflection.Assembly.GetExecutingAssembly().GetName().CodeBase)
            If GetAppPath.StartsWith("file:\") Then
                GetAppPath = GetAppPath.Substring(6)
            End If
        End Function
    End Class

    Public Function SongRenamePattern(ByVal Pattern As String, ByVal Song As SciLorsGroovesharkAPI.Groove.Functions.SearchArtist.SearchArtistResult, Optional ByVal AutoRenameString As String = "") As String
        On Error Resume Next
        Pattern = Pattern.Replace("<artist>", StripIllegalFileName(Song.ArtistName))
        Pattern = Pattern.Replace("<song>", StripIllegalFileName(Song.Name))
        Pattern = Pattern.Replace("<album>", StripIllegalFileName(Song.AlbumName))
        Pattern = Pattern.Replace("<year>", StripIllegalFileName(Song.Year))
        Pattern = Pattern.Replace("<tracknum>", StripIllegalFileName(Song.TrackNum))


        Return StripIllegalPath(Pattern) & AutoRenameString & ".mp3"
    End Function

    Public Function StripIllegalPath(ByVal myString As String) As String
        If myString IsNot Nothing Then
            For Each Character As Char In IO.Path.GetInvalidPathChars
                myString = myString.Replace(Character, "")
            Next
        End If
        Return myString
    End Function

    Public Function StripIllegalFileName(ByVal myString As String) As String
        If myString IsNot Nothing Then
            For Each Character As Char In IO.Path.GetInvalidFileNameChars
                myString = myString.Replace(Character, "")
            Next
        End If
        Return myString
    End Function
End Module
